@@ -25,18 +25,6 @@
 
 #include <glib-object.h>
 
-#include "seahorse-pkcs11-object.h"
-#include "seahorse-pkcs11-source.h"
-
-void          seahorse_pkcs11_refresh_async    (SeahorsePkcs11Source *source,
-                                                GCancellable *cancellable,
-                                                GAsyncReadyCallback callback,
-                                                gpointer user_data);
-
-gboolean      seahorse_pkcs11_refresh_finish   (SeahorsePkcs11Source *source,
-                                                GAsyncResult *result,
-                                                GError **error);
-
 void          seahorse_pkcs11_delete_async     (GList *objects,
                                                 GCancellable *cancellable,
                                                 GAsyncReadyCallback callback,
@@ -45,4 +33,4 @@ void          seahorse_pkcs11_delete_async     (GList *objects,
 gboolean      seahorse_pkcs11_delete_finish    (GAsyncResult *result,
                                                 GError **error);
 
-#endif /* __SEAHORSE_PKCS11_OPERATIONS_H__ */
+#endif /* __SEAHORSE_PKCS11_TOKEN_H__ */
