@@ -22,27 +22,27 @@
 
 #include "config.h"
 
-#include <string.h>
+#include <gcr/gcr.h>
 
 #include <glib.h>
-
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
 #include <errno.h>
+#include <string.h>
 
-#include "seahorse-context.h"
-#include "seahorse-source.h"
+#include "seahorse-place.h"
 #include "seahorse-ssh-source.h"
 #include "seahorse-ssh-key.h"
 #include "seahorse-ssh-operation.h"
-#include "seahorse-gtkstock.h"
+#include "seahorse-icons.h"
 #include "seahorse-validity.h"
 
 enum {
     PROP_0,
     PROP_KEY_DATA,
     PROP_FINGERPRINT,
+    PROP_DESCRIPTION,
     PROP_VALIDITY,
     PROP_TRUST,
     PROP_EXPIRES,
@@ -71,10 +71,14 @@ changed_key (SeahorseSSHKey *self)
 {
 	SeahorseObject *obj = SEAHORSE_OBJECT (self);
 	SeahorseUsage usage;
+	SeahorseFlags flags;
 	const gchar *display = NULL;
 	gchar *identifier;
 	gchar *simple = NULL;
-    
+	GIcon *icon;
+	gchar *filename;
+	gchar *markup;
+
 	if (self->keydata) {
 
   		 /* Try to make display and simple names */
@@ -96,39 +100,53 @@ changed_key (SeahorseSSHKey *self)
 	}
     
 	if (!self->keydata || !self->keydata->fingerprint) {
-        
 		g_object_set (self,
-		              "id", 0,
 		              "label", "",
 		              "icon", NULL,
 		              "usage", SEAHORSE_USAGE_NONE,
 		              "nickname", "",
-		              "location", SEAHORSE_LOCATION_INVALID,
 		              "flags", SEAHORSE_FLAG_DISABLED,
 		              NULL);
 		return;
-		
 	} 
 
-	if (self->keydata->privfile)
+	flags = SEAHORSE_FLAG_EXPORTABLE | SEAHORSE_FLAG_DELETABLE;
+
+	if (self->keydata->privfile) {
 		usage = SEAHORSE_USAGE_PRIVATE_KEY;
-	else
+		flags |= SEAHORSE_FLAG_PERSONAL | SEAHORSE_FLAG_TRUSTED;
+		icon = g_themed_icon_new (GCR_ICON_KEY_PAIR);
+	} else {
+		flags = 0;
 		usage = SEAHORSE_USAGE_PUBLIC_KEY;
+		icon = g_themed_icon_new (GCR_ICON_KEY);
+	}
+
+	if (self->keydata->privfile)
+		filename = g_path_get_basename (self->keydata->privfile);
+	else
+		filename = g_path_get_basename (self->keydata->pubfile);
+
+	markup = g_markup_printf_escaped ("%s<span size='small' rise='0' foreground='#555555'>\n%s</span>",
+	                                  display, filename);
 
 	identifier = seahorse_ssh_key_calc_identifier (self->keydata->fingerprint);
 
+	if (self->keydata->authorized)
+		flags |= SEAHORSE_FLAG_TRUSTED;
+
 	g_object_set (obj,
-	              "id", seahorse_ssh_key_calc_cannonical_id (self->keydata->fingerprint),
+	              "markup", markup,
 	              "label", display,
-	              "icon", SEAHORSE_STOCK_KEY_SSH,
+	              "icon", icon,
 	              "usage", usage,
 	              "nickname", simple,
-	              "location", SEAHORSE_LOCATION_LOCAL,
 	              "identifier", identifier,
-	              "flags", (self->keydata->authorized ? SEAHORSE_FLAG_TRUSTED : 0) | SEAHORSE_FLAG_EXPORTABLE | SEAHORSE_FLAG_DELETABLE,
+	              "flags", flags,
 	              NULL);
-	
+	g_object_unref (icon);
 	g_free (identifier);
+	g_free (markup);
 }
 
 static guint 
@@ -162,7 +180,8 @@ seahorse_ssh_key_get_property (GObject *object, guint prop_id,
                                GValue *value, GParamSpec *pspec)
 {
     SeahorseSSHKey *skey = SEAHORSE_SSH_KEY (object);
-    
+    SeahorseUsage usage;
+
     switch (prop_id) {
     case PROP_KEY_DATA:
         g_value_set_pointer (value, skey->keydata);
@@ -170,6 +189,13 @@ seahorse_ssh_key_get_property (GObject *object, guint prop_id,
     case PROP_FINGERPRINT:
         g_value_set_string (value, skey->keydata ? skey->keydata->fingerprint : NULL);
         break;
+    case PROP_DESCRIPTION:
+        g_object_get (skey, "usage", &usage, NULL);
+        if (usage == SEAHORSE_USAGE_PRIVATE_KEY)
+            g_value_set_string (value, _("Personal SSH key"));
+        else
+            g_value_set_string (value, _("SSH key"));
+        break;
     case PROP_VALIDITY:
         g_value_set_uint (value, calc_validity (skey));
         break;
@@ -240,6 +266,10 @@ seahorse_ssh_key_class_init (SeahorseSSHKeyClass *klass)
         g_param_spec_string ("fingerprint", "Fingerprint", "Unique fingerprint for this key",
                              "", G_PARAM_READABLE));
 
+    g_object_class_install_property (gobject_class, PROP_DESCRIPTION,
+        g_param_spec_string ("description", "Description", "Description",
+                             "", G_PARAM_READABLE));
+
     g_object_class_install_property (gobject_class, PROP_VALIDITY,
         g_param_spec_uint ("validity", "Validity", "Validity of this key",
                            0, G_MAXUINT, 0, G_PARAM_READABLE));
@@ -262,11 +292,14 @@ seahorse_ssh_key_class_init (SeahorseSSHKeyClass *klass)
  */
 
 SeahorseSSHKey* 
-seahorse_ssh_key_new (SeahorseSource *sksrc, SeahorseSSHKeyData *data)
+seahorse_ssh_key_new (SeahorsePlace *place,
+                      SeahorseSSHKeyData *data)
 {
     SeahorseSSHKey *skey;
-    skey = g_object_new (SEAHORSE_TYPE_SSH_KEY, "source", sksrc, 
-                         "key-data", data, NULL);
+    skey = g_object_new (SEAHORSE_TYPE_SSH_KEY,
+                         "place", place,
+                         "key-data", data,
+                         NULL);
     return skey;
 }
 
@@ -312,33 +345,6 @@ seahorse_ssh_key_get_location (SeahorseSSHKey *skey)
                     skey->keydata->privfile : skey->keydata->pubfile;
 }
 
-GQuark
-seahorse_ssh_key_calc_cannonical_id (const gchar *id)
-{
-    #define SSH_ID_SIZE 16
-    gchar *hex, *canonical_id = g_malloc0 (SSH_ID_SIZE + 1);
-    gint i, off, len = strlen (id);
-    GQuark ret;
-
-    /* Strip out all non alpha numeric chars and limit length to SSH_ID_SIZE */
-    for (i = len, off = SSH_ID_SIZE; i >= 0 && off > 0; --i) {
-         if (g_ascii_isalnum (id[i]))
-             canonical_id[--off] = g_ascii_toupper (id[i]);
-    }
-    
-    /* Not enough characters */
-    g_return_val_if_fail (off == 0, 0);
-
-    hex = g_strdup_printf ("%s:%s", SEAHORSE_SSH_STR, canonical_id);
-    ret = g_quark_from_string (hex);
-    
-    g_free (canonical_id);
-    g_free (hex);
-    
-    return ret;
-}
-
-
 gchar*
 seahorse_ssh_key_calc_identifier (const gchar *id)
 {
